﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7.8
@EndOfDesignText@
Sub Class_Globals

	Dim ReportHtml As String
	
End Sub


Public Sub Initialize

End Sub


public Sub AddCss As Boolean

	' If a CSS file exists, leave it alone, otherwise create one.
	Dim filename As String = "unit_tests.css"
	Dim folder As String = File.DirDefaultExternal

	Log("AddCss: folder: " & folder & ", file: " & filename)

	WriteLine($"<link href="${filename}" rel="stylesheet" Type="text/css"
 media="interactive, braille, emboss, handheld, projection, screen, tty, tv, print" />"$)

	If File.Exists(folder, filename) Then
		Log("CSS file found - leaving as is")
		Return True
	End If

	Return CreateDefaultCssFile(folder, filename)

End Sub


Sub CreateDefaultCssFile(folder As String, filename As String) As Boolean

	Log("CreateDefaultCssFile: folder: " & folder & ", file: " & filename)

	Dim css As String = $"
body
{
	background-color: #FFFFFF;
	color: #303090;
	font-family: Verdana, Sans-Serif;
	font-size: 1em;
	margin: 0;
	padding: 0;
}

h1, h2, h3, h4, h5, h6
{
	font-family: Verdana, Sans-Serif;
	font-size: 1em;
	margin-bottom: 10px;
	margin-top: 20px;
}

h1
{
	font-size: 3em;
	xpadding-bottom: 0;
	xmargin-bottom: 0;
}

h2
{
	padding: 0 0 10px 0;
}
"$

	File.WriteString(folder, filename, css)

	Return True

End Sub


public Sub CreateReport(classes As List) As String

	ReportHtml = ""

	WriteLine("<!DOCTYPE html>")
	WriteLine("<html>")

	' Create the head.
	WriteLine("<head>")
	WriteHead
	WriteLine("</head>")

	' Create the body.
	WriteLine("<body>")
	WriteBody(classes)
	WriteLine("</body>")

	WriteLine("</html>")

	Return ReportHtml

End Sub


public Sub WriteBody(classes As List) As Boolean

	' 'classes' is a List of 'TestClassInfoType'.
	WriteLine("<h1>Unit Test Results</h1>")
	WriteLine("")

	WriteLine("App: " & Starter.AppName & ", version " & Starter.AppVersion & "<br />")
	WriteLine("Report created: " & DateTime.Now & "<br />")
	WriteLine("<br />")

	WriteLine("Package: " & Application.PackageName & "<br />")
	WriteLine("Manufacturer: " & Starter.DeviceManufacturer & "<br />")
	WriteLine("Model: " & Starter.DeviceModel & "<br />")
	WriteLine("Device ID: " & Starter.DeviceId & "<br />")
	WriteLine("Android SDK: version " & Starter.AndroidSdkVersion & "<br />")
	WriteLine("<br />")

	Dim nPass As Int = 0
	Dim nFail As Int = 0

	' Get the number of passes and fails.
	For Each classInfo As TestClassInfoType In classes

		For Each methodInfo As TestMethodInfoType In classInfo.Methods

			If methodInfo.LastResult.EqualsIgnoreCase("pass") Then
				nPass = nPass + 1
			Else
				nFail = nFail + 1
			End If

		Next

	Next

	WriteLine($"<p>${nPass} passed, ${nFail} failed</p>"$)

	WriteLine("<table border=""1"">")
	WriteLine("<tr>")
	WriteLine("<th>Class</th><th>Method</th><th>Result</th><th>Time</th><th>Info</th>")
	WriteLine("</tr>")

	Dim passFail As String

	For Each classInfo As TestClassInfoType In classes

		For Each methodInfo As TestMethodInfoType In classInfo.Methods

			If methodInfo.LastResult.EqualsIgnoreCase("pass") Then
				passFail = "<font color=""green"">Pass</font>"
			Else
				passFail = "<font color=""red"">Fail</font>"
			End If

			Dim info As String = methodInfo.Info.Replace("|", "<br />")

			WriteLine("<tr>")
			WriteLine("<td>" & classInfo.ClassName & "</td><td>" & methodInfo.MethodName & "</td><td>" _
				& passFail & "</td><td>" & methodInfo.LastTime & "</td><td>" & info & "</td>")
			WriteLine("</tr>")

		Next

	Next

	WriteLine("</table>")
	WriteLine($"<p>${nPass} passed, ${nFail} failed</p>"$)

	Return True

End Sub


public Sub WriteHead As Boolean

	WriteLine("<title>Test Results</title>")
	AddCss

	Return True

End Sub


public Sub WriteLine(line As String) As Boolean

	ReportHtml = ReportHtml & line & CRLF
	Return True

End Sub

