﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7.8
@EndOfDesignText@
Sub Class_Globals

	Type TestClassInfoType(ClassName As String, Methods As List)

	Type TestMethodInfoType(Class As TestClassInfoType, MethodName As String, Include As Boolean, _
		LastResult As String, LastRun As Long, LastTime As Long, Info As String)

	Private CurrMethod As TestMethodInfoType
	Private CurrStatus As Boolean
	Private NativeMe As JavaObject
	Private ProcessBA As Object

	Public Name As String
	Public NativeActivity As JavaObject
	Public PackageName As String
	Public PackageCodePath As String
	Public Verbosity As Int

End Sub


Public Sub Initialize(a_name As String)

	Log("CTestRunner.Initialize")

	Name = a_name
	NativeActivity = Null
	NativeMe = Me
	PackageName = Application.PackageName
	Verbosity = 9

End Sub


public Sub AssertIntsAreEqual(expected As Int, actual As Int)
	If (expected <> actual) Then
		TestFailed("AssertIntsAreEqual: Expected=" & expected & ", Actual=" & actual)
	End If
End Sub


public Sub AssertIntsAreEqual2(expected As Int, actual As Int, message As String)
	If (expected <> actual) Then
		TestFailed("AssertIntsAreEqual: Expected=" & expected & ", Actual=" & actual & " (" & message & ")")
	End If
End Sub


public Sub AssertIntsDiffer(expected As Int, actual As Int)
	If (expected = actual) Then
		TestFailed("AssertIntsDiffer: Expected=" & expected & ", Actual=" & actual)
	End If
End Sub


public Sub AssertIntsDiffer2(expected As Int, actual As Int, message As String)
	If (expected = actual) Then
		TestFailed("AssertIntsDiffer: Expected=" & expected & ", Actual=" & actual & " (" & message & ")")
	End If
End Sub


public Sub AssertIsFalse(value As Object)
	If (value = Null) Then
		TestFailed("AssertIsFalse: Actual=null")
	End If

	Dim boolValue As Boolean = value

	If boolValue Then
		TestFailed("AssertIsFalse: Actual=true")
	End If
End Sub


public Sub AssertIsFalse2(value As Object, message As String)
	If (value = Null) Then
		TestFailed("AssertIsFalse: Actual=null (" & message & ")")
	End If

	Dim boolValue As Boolean = value

	If boolValue Then
		TestFailed("AssertIsFalse: Actual=true (" & message & ")")
	End If
End Sub


public Sub AssertIsNotNull(value As Object)
	If (value = Null) Then
		TestFailed("AssertIsNotNull: Actual=" & value)
	End If
End Sub


public Sub AssertIsNotNull2(value As Object, message As String)
	If (value = Null) Then
		TestFailed("AssertIsNotNull: Actual=" & value & " (" & message & ")")
	End If
End Sub


public Sub AssertIsNull(value As Object)
	If (value <> Null) Then
		TestFailed("AssertIsNull: Actual=" & value)
	End If
End Sub


public Sub AssertIsNull2(value As Object, message As String)
	If (value <> Null) Then
		TestFailed("AssertIsNull: Actual=" & value & " (" & message & ")")
	End If
End Sub


public Sub AssertIsTrue(value As Object)
	If (value = Null) Then
		TestFailed("AssertIsTrue: Actual=null")
	End If

	Dim boolValue As Boolean = value

	If Not(boolValue) Then
		TestFailed("AssertIsTrue: Actual=false")
	End If
End Sub


public Sub AssertIsTrue2(value As Object, message As String)
	If (value = Null) Then
		TestFailed("AssertIsTrue: Actual=null (" & message & ")")
	End If

	Dim boolValue As Boolean = value

	If Not(boolValue) Then
		TestFailed("AssertIsTrue: Actual=false (" & message & ")")
	End If
End Sub


public Sub AssertStringsAreEqual(expected As String, actual As String)
	If (expected = Null) Then
		TestFailed("AssertStringsAreEqual: Expected=null")
	End If

	If (actual = Null) Then
		TestFailed("AssertStringsAreEqual: Actual=null")
	End If

	If (expected <> actual) Then
		TestFailed("AssertStringsAreEqual: Expected='" & expected & "', Actual='" & actual & "'")
	End If
End Sub


public Sub AssertStringsAreEqualIgnoreCase(expected As String, actual As String)
	If (expected = Null) Then
		TestFailed("AssertStringsAreEqual: Expected=null")
	End If

	If (actual = Null) Then
		TestFailed("AssertStringsAreEqual: Actual=null")
	End If

	If (expected.ToUpperCase <> actual.ToUpperCase) Then
		TestFailed("AssertStringsAreEqualIgnoreCase: Expected='" & expected & "', Actual='" & actual & "'")
	End If
End Sub


public Sub AssertStringsDiffer(expected As String, actual As String)
	If (expected = Null) Then
		TestFailed("AssertStringsDiffer: Expected=null")
	End If

	If (actual = Null) Then
		TestFailed("AssertStringsDiffer: Actual=null")
	End If

	If (expected = actual) Then
		TestFailed("AssertStringsDiffer: Expected='" & expected & "', Actual='" & actual & "'")
	End If
End Sub


public Sub AssertStringsDifferIgnoreCase(expected As String, actual As String)
	If (expected = Null) Then
		TestFailed("AssertStringsDifferIgnoreCase: Expected=null")
	End If

	If (actual = Null) Then
		TestFailed("AssertStringsDifferIgnoreCase: Actual=null")
	End If

	If (expected.ToUpperCase = actual.ToUpperCase) Then
		TestFailed("AssertStringsDifferIgnoreCase: Expected='" & expected & "', Actual='" & actual & "'")
	End If
End Sub


public Sub Discover(classPrefix As String, subPrefix As String) As List

	' Returns a List of 'TestClassInfoType'.
	Dim result As List
	result.Initialize

	PackageCodePath = NativeActivity.RunMethod("GetPackageCodePath", Null)
	ProcessBA = NativeActivity.RunMethod("GetProcessBA", Null)

	' Get all class names in this package.
	Dim classNames As List = NativeMe.RunMethod("GetClassNamesInPackage", _
		Array(PackageName, PackageCodePath, Verbosity > 3))
	classNames.SortCaseInsensitive(True)

	For Each className As String In classNames

		If className.StartsWith("ctestrunner") Or className.Contains("$ResumableSub") _
			Or Not(className.StartsWith(classPrefix)) Then
			If Verbosity > 2 Then Log("CLASS: " & className & " - ignored")
			Continue
		End If

		Dim classInfo As TestClassInfoType
		classInfo.Initialize
		classInfo.ClassName = className
		classInfo.Methods.Initialize
		result.Add(classInfo)

		If Verbosity > 1 Then Log("CLASS: " & className)

		' Get all method names in the Java class.
		Dim javaClassName As String = PackageName & "." & className.ToLowerCase()

		Dim methodNames As List = NativeMe.RunMethod("GetMethodNames", Array(javaClassName, Verbosity > 2))
		methodNames.SortCaseInsensitive(True)

		For Each methodName As String In methodNames

			If methodName.StartsWith("_" & subPrefix) Then
				Dim methodInfo As TestMethodInfoType
				methodInfo.initialize()
				methodInfo.Class = classInfo
				methodInfo.Include = True
				methodInfo.Info = ""
				methodInfo.LastResult = "-"
				methodInfo.LastRun = 0
				methodInfo.MethodName = methodName.SubString(1)
				classInfo.Methods.Add(methodInfo)
			Else
				If Verbosity > 2 Then
					Log("  Method: " & methodName & " - ignored")
				End If
			End If

		Next

	Next

	Return result

End Sub


public Sub RunAll(classes As List, baseClass As String, classPrefix As String, subPrefix As String, _
	failsOnly As Boolean) As Boolean

	' Test all classes in 'classes' (a List of 'TestClassInfoType') with names beginning with 'classPrefix',
	' and all methods beginning with 'subPrefix'.
	' If 'baseClass' is specified and exists:
	'		an optional overall 'Setup' sub is called before any Tests are run.
	'		an optional overall 'Teardown' sub is called after all Tests have been run.
	' Returns 'True' if all tests passed.

	Dim result As Boolean = True

	' Overall Setup?
	RunOne(baseClass, "Setup")

	Dim oneArgInvoke As Boolean = Not(Starter.Debugging) Or Starter.LegacyDebugging

	For Each classInfo As TestClassInfoType In classes

		Dim className As String = classInfo.ClassName

		If className.StartsWith("ctestrunner") _
			Or className.StartsWith("ctestreporter") _
			Or className.Contains("$ResumableSub") _
			Or className.EqualsIgnoreCase(baseClass) _
			Or Not(className.StartsWith(classPrefix)) Then

			If Verbosity > 2 Then Log("CLASS: " & className & " - ignored")
			Continue
		End If

		If Verbosity > 1 Then Log("CLASS: " & className)

		' Create an object of class 'classname'.
		Dim javaClassName As String = PackageName & "." & className.ToLowerCase()

		Dim obj As Object = NativeMe.RunMethod("InstantiateClass", Array(javaClassName, Verbosity > 2))

		If (obj = Null) Then
			Log("ERROR - Couldn't instantiate class: " & className)
			Continue
		End If

		' Initialize the new object.
		Dim status As Boolean = NativeMe.RunMethod("InitializeB4Object", _
			Array(obj, ProcessBA, oneArgInvoke, Verbosity > 2))

		If Not(status) Then
			Log("ERROR - Couldn't initialize object of class: " & className)
			Continue
		End If

		' Run each Test method.
		Dim failed As Boolean
		Dim methodName As String
		Dim strStatus As String

		For Each methodInfo As TestMethodInfoType In classInfo.Methods

			' Test this method?
			If Not(methodInfo.Include) Then Continue

			failed = (methodInfo.LastRun > 0) And methodInfo.LastResult.EqualsIgnoreCase("fail")
			If failsOnly And Not(failed) Then Continue

			methodName = methodInfo.MethodName
			If StringHelpers.IsNullOrWhiteSpace(methodName) Then Continue

			If Not(methodName.StartsWith(subPrefix)) Then
				If Verbosity > 2 Then
					Log("  Method: " & methodName & " - ignored")
				End If

				Continue
			End If

			' Yes, test this method.
			CurrMethod = methodInfo
			CurrStatus = True
			If Verbosity > 1 Then Log("  Method: " & methodName)

			' Setup?
			If SubExists(obj, "Setup") Then
				CallSub2(obj, "Setup", methodName)
			End If

			methodInfo.Info = ""
			methodInfo.LastRun = DateTime.Now

			Try
				strStatus = NativeMe.RunMethod("InvokeMethod", Array(obj, "_" & methodName, oneArgInvoke, Verbosity > 2))
			Catch
				Log("ERROR - " & LastException.Message)

				If (methodInfo.Info <> "") Then methodInfo.Info = methodInfo.Info & "|"
				methodInfo.Info = methodInfo.Info & LastException.Message
			End Try

			methodInfo.LastTime = DateTime.Now - methodInfo.LastRun

			If (CurrStatus And (strStatus = "")) Then
				' PASSED.
				methodInfo.LastResult = "pass"
			Else
				' FAILED.
				methodInfo.LastResult = "FAIL"

				If (methodInfo.Info <> "") Then methodInfo.Info = methodInfo.Info & "|"
				methodInfo.Info = methodInfo.Info & strStatus

				result = False
			End If

			' Teardown?
			If SubExists(obj, "Teardown") Then
				CallSub2(obj, "Teardown", methodName)
			End If

		Next

	Next

	' Overall Teardown?
	RunOne(baseClass, "Teardown")

	Return result

End Sub


public Sub RunOne(className As String, methodName As String) As Boolean
	
	' Run the sub specified by 'className' / 'methodName'.
	' Used to call overall 'Setup' and 'Teardown' methods.
	If Verbosity > 1 Then Log("RunOne: Class " & className & ", method " & methodName)

	' Create an object of class 'className'.
	Dim javaClassName As String = PackageName & "." & className.ToLowerCase()

	Dim obj As Object = NativeMe.RunMethod("InstantiateClass", Array(javaClassName, Verbosity > 2))

	If (obj = Null) Then
		Log("ERROR - Couldn't instantiate class: " & className)
		Return False
	End If

	' Initialize the new object.
	Dim oneArgInvoke As Boolean = Not(Starter.Debugging) Or Starter.LegacyDebugging

	Dim status As Boolean = NativeMe.RunMethod("InitializeB4Object", _
		Array(obj, ProcessBA, oneArgInvoke, Verbosity > 2))

	If Not(status) Then
		Log("ERROR - Couldn't initialize object of class: " & className)
		Return False
	End If

	If Not(SubExists(obj, methodName)) Then
		Log("ERROR - Class '" & className & "' has no method: " & methodName)
		Return False
	End If

	Return CallSub(obj, methodName)

End Sub


private Sub TestFailed(message As String)
	
	If Not(StringHelpers.IsNullOrWhitespace(message)) Then
		CurrMethod.Info = message
		Log(message)
	End If

	CurrStatus = False

	Dim ex As ExceptionEx
	ex.Initialize("")
	ex.Throw

End Sub


#If JAVA
	import java.lang.reflect.*;
	import java.util.Enumeration;
	import java.util.Iterator;
	import java.util.ArrayList;
	import java.util.Vector;
	import dalvik.system.DexFile;


	public ArrayList<String> GetClassNamesInPackage(String packageName, String packageCodePath,
		boolean logIt)
	{
		if (logIt)
			BA.Log("GetClassNamesInPackage: " + packageName + " (path " + packageCodePath + ")");

		ArrayList<String> classNames = new ArrayList<String>();

		try
		{
			DexFile df = new DexFile(packageCodePath);

			for (Enumeration<String> iter = df.entries(); iter.hasMoreElements(); )
			{
				String className = iter.nextElement();

				if (className.startsWith(packageName))
				{
					className = className.substring(className.lastIndexOf(".") + 1, className.length());
					classNames.add(className);

					if (logIt)
						BA.Log("Class: " + className);
				}
			}
		}
		catch (Throwable e)
		{
			BA.Log("ERROR - " + e.toString());
		}

		return classNames;
	}


	private Method GetMethod(Class c, String methodName, boolean LogIt)
	{
		Method result = null;

		try
		{
			// NO: method = c.getMethod(methodName, null);

			// Doing this the long way, and ignoring the method signature.
			Method[] m = c.getDeclaredMethods();

			for (int i = 0; i < m.length; i++)
			{
				if (m[i].getName().equals(methodName))
				{
					result = m[i];
					break;
				}
			}
		}
		catch (Throwable e)
		{
			BA.Log("ERROR - Failed to get method " + methodName + ": " + e.toString());
			return null;
		}

		if (result == null)
		{
			BA.Log("ERROR - Failed to find method " + methodName);
			return null;
		}

		return result;
	}
	

	public ArrayList<String> GetMethodNames(String className, boolean logIt)
	{
		if (logIt)
			BA.Log("GetMethodNames: " + className);

		ArrayList<String> methodNames = new ArrayList<String>();

		// Get the Java class.
		Class c;

		try
		{
			c = Class.forName(className);
		}
		catch (Throwable e)
		{
			BA.Log("ERROR - No such Java class: " + className + ": " + e.toString());
			return null;
		}

		// Get the methods.
		try
		{
			Method[] m = c.getDeclaredMethods();

			if (logIt)
				BA.Log("" + m.length + " methods");

			for (int i = 0; i < m.length; i++)
			{
				if (m[i].getName() != null)
					methodNames.add(m[i].getName());
			}
		}
		catch (Throwable e)
		{
			BA.Log("ERROR - " + e.toString());
		}

		return methodNames;
	}


	public boolean InitializeB4Object(Object obj, Object processBA, boolean oneArgInvoke, boolean logIt)
	{
		if (logIt)
			BA.Log("InitializeB4Object: " + obj.getClass().getName());

		try
		{
			Method method = GetMethod(obj.getClass(), "_initialize", logIt);

			if (method == null)
			{
				BA.Log("ERROR - No _initialize method!");
				return false;
			}

			Object returnObj = null;

			if (oneArgInvoke)
				returnObj = method.invoke(obj, processBA);
			else
				returnObj = method.invoke(obj, null, processBA);
		}
		catch (Throwable e)
		{
			BA.Log("ERROR - Failed to initialize object: " + e.toString());
			BA.Log("  Cause: " + e.getCause().toString());
			return false;
		}
		
		return true;
	}


	public Object InstantiateClass(String className, boolean logIt)
	{
		if (logIt)
			BA.Log("InstantiateClass: " + className);

		Object result = null;

		// Get the Java class.
		Class c;

		try
		{
			c = Class.forName(className);
		}
		catch (Throwable e)
		{
			BA.Log("ERROR - No such Java class: " + className);
			return null;
		}

		// Create an object.
		Object o;

		try
		{
			result = c.newInstance();

			if (logIt)
				BA.Log("Created a " + result.getClass().getSimpleName());
		}
		catch (Throwable e)
		{
			BA.Log("ERROR - Failed to instantiate class " + className + "': " + e.toString());
			return null;
		}

		return result;
	}


	public String InvokeMethod(Object obj, String methodName, boolean oneArgInvoke, boolean logIt)		//throws Exception
	{
		{
			String result = "";

			if (logIt)
				BA.Log("InvokeMethod: " + obj.getClass().getName() + ", " + methodName);

			try
			{
				Method method = GetMethod(obj.getClass(), methodName, logIt);

				if (method == null)
				{
					result = "ERROR: Failed to get method: " + methodName;
					return result;
				}

				Object returnObj;

				if (oneArgInvoke)
					returnObj = method.invoke(obj);
				else
					returnObj = method.invoke(obj, obj);
			}
			catch (Throwable e)
			{
				if (logIt)
				{
					BA.Log("ERROR: " + e.getCause().toString());
					BA.Log("Failed to invoke method " + methodName + ": " + e.toString());
				}

				result = "ERROR: " + e.getCause().toString();
				result += "|" + "Failed to invoke method " + methodName + ": " + e.toString();
			}
			
			return result;
		}
	}

#End If

